# -*- coding: utf-8 -*-

import os
import json
from google.cloud import pubsub
from pymongo import MongoClient
import scrapy
from scrapy.selector import Selector
import logging
logger = logging.getLogger(__name__)

class MongoPipeline(object):
    @classmethod
    def from_crawler(cls, crawler):
        return cls(settings = crawler.settings)


    def __init__(self, settings):
        self.mongo_uri = settings.get('MONGO_URI'),
        self.mongo_db = settings.get('MONGO_DATABASE', 'ze-the-scraper')
        self.client = None


    def open_spider(self, spider):
        self.client = MongoClient(self.mongo_uri)
        self.db = self.client[self.mongo_db]


    def close_spider(self, spider):
        self.client.close()


    def process_item(self, item, spider):
        # TODO: Get collection name via item name
        self.db['NewsArticle'].insert(dict(item))
        return item


class GooglePubSubPipeline(object):
    @classmethod
    def from_crawler(cls, crawler):
        return cls(crawler.settings, crawler.stats)
 

    def __init__(self, settings, stats):
        self.google_cloud_enabled = settings.getbool('GOOGLE_CLOUD_ENABLED')
        self.settings = {
            'project_name': settings.get('PROJECT_NAME')
        }
        self.stats = stats
        self.topics = {}
        self.stats.set_value('google/pubsub/published_count', 0)
        self.stats.set_value('google/pubsub/erros_count', 0)

        if self.google_cloud_enabled:
            self.client = pubsub.Client()
            logger.info('Google Cloud Pub/Sub client initiated with success')
        else:
            logger.error('Google Cloud is not enabled, check Google Cloud extension configuration')


    def open_spider(self, spider):
        if self.google_cloud_enabled:
            try:
                # TODO: Use multiples topics {} and create topic for using spider and item nam
                self.topic = self.client.topic('ze-the-scraper.'+spider.name+'.newsarticle')
                if not self.topic.exists():
                    self.topic.create()
            except Exception as e:
                logger.error('Failed to get or create topic in Google Cloud Pub/Sub: %s' % e)


    def process_item(self, item, spider):
        if self.google_cloud_enabled:
            try:
                topic_name = '%s.%s.%s' % (self.settings['project_name'], spider.name, 'newsarticle')
                topic = None
                
                if topic_name in self.topics:
                    topic = self.topics[topic_name] 
                else:
                    self.topics[topic_name] = self.client.topic(topic_name)
                    
                    if not self.topics[topic_name].exists():
                        self.topics[topic_name].create()
                    
                    topic = self.topics[topic_name]
                
                topic.publish(json.dumps(dict(item)))
                
                self.stats.inc_value('google/pubsub/published_count')
            except Exception as e:
                logger.error('Failed publish item to Google Cloud Pub/Sub: %s' % e)
                self.stats.inc_value('google/pubsub/erros_count')
        return item



# -*- coding: utf-8 -*-

import scrapy
from scrapy.loader import ItemLoader
from scrapy.spiders import Rule
from scrapy.linkextractors import LinkExtractor as sle
import ze
from ze.items.newsarticle import NewsArticleItem, NewsArticleItemLoader


class VejaSpider(ze.spiders.ZeSpider):
    name = 'veja'
    allowed_domains = ['veja.abril.com.br']
    start_urls = []
    settings = {
        'search_url': 'http://veja.abril.com.br/?infinity=scrolling'
    }
    

    def __init__(self, *args, **kwargs):
        super(VejaSpider, self).__init__(*args, **kwargs)
        self.args = kwargs
        print(self.args)
        if self.args.get('url') and self.args.get('search'):
            raise ValueError('url and search arguments is passed, must one of then')
        elif self.args.get('url'):
            self.start_urls = [self.args['url']]

    def start_requests(self):
        urls = []
        if self.args['search']['engine'] in ['google']:
            self.args['search']['params']['query'] = '%s site:%s' % (
                self.args['search']['params']['query'], 
                self.allowed_domains[0])
            urls = self.get_urls_from_search_engine(
                self.args['search']['engine'], 
                self.args['search']['params'])
        
            for url in urls:
                yield scrapy.Request(url, callback=self.parse_article)
        elif self.args['search']['engine'] == 'own':
            raise NotImplementedError
        else:
            raise ValueError('search.provider is not valid, please search `google` or `own`')

    def get_urls_from_search(self, params):
        raise NotImplementedError
        formdata = {
            'action': 'infinite_scroll',
            'page': '1',
            'currentday': '29.03.17',
            'order': 'DESC',
            'query_args[paged]': '0',
            'query_args[category_name]': 'educacao',
            'query_args[p]': '0',
            'query_args[second]': '',
            'query_args[minute]': '',
            'query_args[hour]': '',
            'query_args[day]': '',
            'query_args[monthnum]': '0',
            'query_args[year]': '0',
            'query_args[w]': '0',
            'query_args[tag]': '',
            'query_args[cat]': '515646675',
            'query_args[tag_id]': '',
            'query_args[author]': '',
            'query_args[author_name]': '',
            'query_args[feed]': '',
            'query_args[sentence]': '',
            'query_args[title]': '',
            'query_args[fields]': '',
            'query_args[post_type][0]': 'post',
            'query_args[post_type][1]': 'video',
            'query_args[post_type][2]': 'blog_post',
            'query_args[post_type][3]': 'special_post',
            'query_args[posts_per_page]': '10',
            'query_args[cache_results]': 'false',
            'query_args[update_post_term_cache]': 'true',
            'query_args[lazy_load_term_meta]': 'false',
            'query_args[update_post_meta_cache]': 'true',
            'query_args[nopaging]': 'false',
            'query_args[comments_per_page]': '10',
            'query_args[no_found_rows]': 'false',
            'query_args[order]': 'DESC',
            'query_args[extra][is_tag]': 'false',
            'query_args[extra][is_category]': 'true',
            'query_args[extra][is_admin]': 'false',
            'last_post_date': '2017-03-19 12:24:15',
        }

    def parse_article(self, response):
        l = NewsArticleItemLoader(item=NewsArticleItem(), response=response)

        l.add_css('name', '[itemprop=headline]::text')
        l.add_fallback_css('name', '.article-title::text')
        l.add_css('author', '[itemprop=author]::text')
        l.add_fallback_css('author', '.article-author span strong::text')
        l.add_css('description', '[itemprop=description]::text')
        l.add_fallback_css('description', '.article-subtitle::text')
        l.add_css('datePublished', '[itemprop=datePublished]::text')
        l.add_fallback_css('datePublished', '.article-date span::text')
        l.add_css('dateModified', '[itemprop=dateModified]::text')
        l.add_fallback_css('dateModified', '.article-date span::text')
        l.add_css('keywords', '[itemprop=keywords] a::text')
        l.add_fallback_css('keywords', '.article-tags a::text')
        l.add_css('articleBody', '[itemprop=articleBody]')
        l.add_fallback_css('articleBody', '.article-content')
        l.add_value('url', response.url)
        
        if 'blog' in response.url:
            l.add_value('sources_types', ('portal', 'blog'))
        else:
            l.add_value('sources_types', ('portal'))

        yield l.load_item()


# -*- coding: utf-8 -*-

import re
import logging
from bs4 import BeautifulSoup, Comment

logger = logging.getLogger(__name__)

class CleanHTML(object):

    def __call__(self, value, context=None):
        soup = BeautifulSoup(value, 'html.parser')
        print(soup.prettify())
        print('##############################################')
        print('##############################################')
        print('##############################################')
        
        # veja
        for e in soup.select('.featured-image'):
            f = soup.new_tag('figure')
            f.append(soup.new_tag('img', src=e.select('img')[0]['data-src']))
            fc = soup.new_tag('figcaption')
            fc.string = string=e.select('p')[0].string
            f.append(fc)
            
            e.replace_with(f)
        
        [e.unwrap() for e in soup.select('section.article-content')]
        
        [e.decompose() for e in soup.select('div.widget-news')]
        
        # g1
        for e in soup.select('[data-block-type="backstage-photo"]'):
            fg = soup.new_tag('figure')
            fg.append(soup.new_tag('img', src=e.select('img.content-media__image')[0]['data-src']))
            fc = soup.new_tag('figcaption')
            fc.string = e.select('img.content-media__image')[0]['alt']
            f.append(fc)
            
            e.replace_with(fg)
        
        for e in soup.select('[data-block-type="backstage-video"]'):
            video_id = e.select('[data-video-id]')[0]['data-video-id']
            fg = soup.new_tag('figure')
            fg.append(soup.new_tag('img', src='https://s02.video.glbimg.com/x720/%s.jpg' % video_id))
            fc = soup.new_tag('figcaption')
            fc.string = e.select('[itemprop="caption"]')[0]['content']
            f.append(fc)
            a = soup.new_tag('a', href='https://globoplay.globo.com/v/%s/' 
                % video_id)
            a.append(fg)
            
            e.replace_with(a)
        
        # cartacapital
        for e in soup.select('p iframe[data-lazy-src*="https://www.youtube.com/embed"]'):
            # TODO: Use Regex?
            video_id = e['data-lazy-src'].split('/')[4]
            fm = soup.new_tag('iframe', src='https://www.youtube.com/embed/%s?rel=0' % video_id,
                width='1280', height='720', frameborder='0', allowfullscreen='true')
            
            e.parent.replace_with(fm)
        
        # All
        [e.unwrap() for e in soup.select("""
            [itemprop="articleBody"], 
            main, 
            .content-text, 
            .content-intertitle, 
            .td-post-content, 
            .td-post-featured-image, 
            #mobile1stparagraph, 
            figure a""")]
        
        [e.decompose() for e in soup.select("""
            .content-head, 
            style, 
            .content-share-bar, 
            .mc-side-item__container, 
            .mc-show-later, 
            .content-share-bar, 
            .content-ads, 
            [id^="ad-"], 
            .comments""")]
        # Remove empy elements
        [e.decompose() for e in soup.select('p, div') if not e.contents]
        
        # TODO: B4S bug
        [e.previous_element.decompose() for e in soup.select('p + br + p')]
        
        for e in soup.select('*'):
            del e['class']
            del e['data-track-category']
            del e['data-track-links']
            del e['width']
            del e['height']
        
        for e in soup.select('img'):
            if e.get('alt').strip(): del e['alt']
            del e['title']
        
        for comments in soup.findAll(text=lambda text:isinstance(text, Comment)):
            comments.extract()
        
        value = soup.prettify()
        print(value)
        return value


# -*- coding: utf-8 -*-

# Define here the models for your scraped items
#
# See documentation in:
# http://doc.scrapy.org/en/latest/topics/items.html

import scrapy
from scrapy.loader import ItemLoader
from scrapy.loader.processors import TakeFirst, MapCompose
from ze.processors.article import ArticleProcessor
from ze.processors.common import CommonProcessor
from ze.processors.html import CleanHTML

class NewsArticleItem(scrapy.Item):
    name = scrapy.Field(
        output_processor=TakeFirst(),
    )
    datePublished = scrapy.Field(
        input_processor=MapCompose(
            CommonProcessor.process_date_time
        ),
        output_processor=TakeFirst(),
    )
    dateModified = scrapy.Field(
        input_processor=MapCompose(
            CommonProcessor.process_date_time
        ),
        output_processor=TakeFirst(),
    )
    description = scrapy.Field(
        output_processor=TakeFirst(),
    )
    author = scrapy.Field(
        input_processor=MapCompose(
            ArticleProcessor.process_authors
        )
    )
    articleBody = scrapy.Field(
        input_processor=MapCompose(
            CleanHTML(),
        ),
        output_processor=TakeFirst(),
    )
    image = scrapy.Field()
    keywords = scrapy.Field(
        input_processor=MapCompose(
            ArticleProcessor.process_keywords
        )
    )
    commentCount = scrapy.Field(
        output_processor=TakeFirst(),
    )
    url = scrapy.Field(
        output_processor=TakeFirst(),
    )
    sources_types = scrapy.Field()


class NewsArticleItemLoader(ItemLoader):
    def get_collected_values(self, field_name):
        return (self._values[field_name]
                if field_name in self._values
                else self._values.default_factory())

    def add_fallback_css(self, field_name, css, *processors, **kw):
        if not any(self.get_collected_values(field_name)):
            self.add_css(field_name, css, *processors, **kw)

    def add_fallback_xpath(self, field_name, css, *processors, **kw):
        if not any(self.get_collected_values(field_name)):
            self.add_xpath(field_name, css, *processors, **kw)



# -*- coding: utf-8 -*-

import re

class ArticleProcessor():

    @staticmethod
    def process_authors(value):
        strings_to_clean = ('O Estado de S.Paulo', 'O Estado de S. Paulo',
        u'estadão.com.br', 'por ', 'Por ', 'RIO', '\n', '\t', '\"', '*',
        ' - ', '/', ',')
        # TODO: Tratar essa página
        # http://politica.estadao.com.br/noticias/panama-papers,panama-papers-revelam-107-offshores-ligadas-a-personagens-da-lava-jato,10000024501
        for string in strings_to_clean:
            value = value.replace(string, '')

        return [a.strip()
                for a in re.split(', | e ', value)]

    @staticmethod
    def process_keywords(value):
        return value.strip().replace(' -', '').replace(',', '').lower()



# -*- coding: utf-8 -*-

def load_lines(path):
    with open(path, 'rb') as f:
        return [line.strip() for line in
                f.read().decode('utf8').splitlines()
                if line.strip()]


# -*- coding: utf-8 -*-
import copy
import subprocess
import json
import six
import os

import scrapy
from scrapy.http import Request, HtmlResponse
from scrapy.utils.spider import iterate_spider_output

import GoogleScraper

class ZeSpider(scrapy.Spider):
    
    @classmethod
    def from_crawler(cls, crawler, *args, **kwargs):
        for key, value in kwargs.items():
            try:
                kwargs[key] = json.loads(value)
            except Exception:
                # TODO: Add debug
                pass
        spider = cls(*args, **kwargs)
        spider._set_crawler(crawler)
        return spider
    
    def get_urls_from_search_engine(self, search_engine='google', args={}):
        config = {}
        
        # TODO: implement quantity arg
        if search_engine == 'google':
            config = {
                'use_own_ip': 'True',
                'keywords': [args['query']],
                'google_search_url': 'https://www.google.com/search?qdr:%s&' % args.get('time_range', 'w'),
                'num_results_per_page': 10,
                'num_pages_for_keyword': 1,
                'num_workers': 1,
                'search_engines': ['google',],
                'search_type': 'normal',
                'scrape_method': 'http',
                'do_caching': False,
                'print_results': None,
            }
        else:
            raise NotImplementedError('Only Google serch engine is supported at momment')
        
        self.logger.info('Google Search scrapping start with this configuration: %s' % config)
        
        try:
            search = GoogleScraper.scrape_with_config(config)
        except GoogleScraper.GoogleSearchError as e:
            self.logger.error(str(e))
        
        urls = []
        for serp in search.serps:
            [urls.append(r.link) for r in serp.links]
        
        self.logger.info('Google Search scrapped with success: %d links extracted' % len(urls))
        self.logger.info('List of link extracted from Google Search: %s' % urls)
        return urls


# -*- coding: utf-8 -*-

import math
try:
    import urlparse
    from urllib import urlencode
except: # Python 3
    import urllib.parse as urlparse
    from urllib.parse import urlencode
import scrapy
from scrapy.spiders import CrawlSpider, Rule
from scrapy.loader import ItemLoader
from scrapy.exceptions import CloseSpider
from ze.items.newsarticle import NewsArticleItem, NewsArticleItemLoader

class EstadaoArticlesSpider(scrapy.Spider):

    name = 'g1'
    allowed_domains = ['g1.globo.com']
    config = {
        'searchlUrl': 'http://g1.globo.com/busca/',
        'cdnUrl': 'http://falkor-cda.bastian.globo.com/feeds/{feedId}/posts/page/{page}',
        'args': None,
        'feeds': {
            'educacao': '42f93818-a7ce-41d9-9a3f-57f43df62f03'
        }
    }


    def __init__(self, url=None, query=None, editorial=None, subject=None, when=None, contentType=None, *args, **kwargs):
        super(EstadaoArticlesSpider, self).__init__(*args, **kwargs)
        
        self.config['args'] = {
            'source': 'url', 
            'url': url, 
            'query': query, 
            'editorial': editorial, 
            'subject': subject, 
            'when': when, 
            'contentType': contentType, 
        }
        
        if url:
            self.start_urls = [self.config['args']['url']]
        else:
            self.config['args']['source'] = 'defaultSearchlUrl'
            self.start_urls = [self.generateUrl(self.config['args'])]


    def parse(self, response):
        if self.config['args']['source'] == 'url':
            yield scrapy.Request(self.config['args']['url'], callback=self.parse_article)
        
        if self.config['args']['source'] == 'defaultSearchlUrl':
            results = [int(s) for s in response.css('.flt-registros::text').extract()[0].split() if s.isdigit()][0]
            self.logger.info('Total of Results: %d', results)
            pages = int(math.ceil(results / 10))
            self.logger.info('Total of Pages: %d', pages)
            
            self.config['args']['source'] = 'ajaxSearchUrl'
            
            for p in range(1, pages+1):
                self.config['args']['page'] = p
                yield scrapy.Request(self.generateUrl(self.config['args']), callback=self.parse_article_urls)


    def parse_article_urls(self, response):
        for i, url in enumerate(response.css('.link-title::attr(href)').extract()):
            yield scrapy.Request(url, meta={'index': i}, callback=self.parse_article)


    def parse_article(self, response):
        l = NewsArticleItemLoader(item=NewsArticleItem(), response=response)

        l.add_css('name', '[itemprop=headline]::text')
        l.add_fallback_css('name', '.content-head__title::text')
        l.add_css('author', '[itemprop=creator]::text')
        # l.add_css('publisher', '[itemprop=publisher]::attr(content)')
        l.add_css('description', '[itemprop=alternativeHeadline]::text')
        l.add_fallback_css('description', '.content-head__subtitle::text')
        l.add_css('datePublished', '[itemprop=datePublished]::attr(datetime)')
        l.add_css('dateModified', '[itemprop=dateModified]::attr(datetime)')
        l.add_css('articleBody', '.mc-body')
        l.add_css('keywords', '.entities__list-itemLink::text')
        l.add_value('url', response.url)
        
        if 'blog' in response.url:
            l.add_value('sources_types', ('portal', 'blog'))
        else:
            l.add_value('sources_types', ('portal'))

        yield l.load_item()


    def generateUrl(self, values):
        url_parts = list(urlparse.urlparse(self.config[values['source']]))
        params = {}
        
        if values['query']:
            params['q'] = (values['query'])
        if values['editorial']:
            params['editoria[]'] = (values['editorial'])
        if values['subject']:
            params['assunto[]'] = (values['subject'])
        if values['when']:
            params['quando'] = (values['when'])
        if values['contentType']:
            params['tipo_conteudo'] = (values['contentType'])
        
        url_parts[4] = urlencode(params)
        
        return urlparse.urlunparse(url_parts)



# -*- coding: utf-8 -*-

import math
try:
    import urlparse
    from urllib import urlencode
except: # Python 3
    import urllib.parse as urlparse
    from urllib.parse import urlencode
import scrapy
from scrapy.spiders import CrawlSpider, Rule
from scrapy.loader import ItemLoader
from scrapy.exceptions import CloseSpider
from ze.items.newsarticle import NewsArticleItem, NewsArticleItemLoader

class EstadaoArticlesSpider(scrapy.Spider):

    name = 'uol'
    allowed_domains = ['uol.com.br']
    config = {
        'args': None
    }


    def __init__(self, url=None, query=None, editorial=None, subject=None, when=None, contentType=None, *args, **kwargs):
        super(EstadaoArticlesSpider, self).__init__(*args, **kwargs)
        
        self.config['args'] = {
            'source': 'url', 
            'url': url, 
            'query': query, 
            'editorial': editorial, 
            'subject': subject, 
            'when': when, 
            'contentType': contentType, 
        }
        
        if url:
            self.start_urls = [self.config['args']['url']]
        else:
            self.config['args']['source'] = 'defaultSearchlUrl'
            self.start_urls = [self.generateUrl(self.config['args'])]


    def parse(self, response):
        if self.config['args']['source'] == 'url':
            yield scrapy.Request(self.config['args']['url'], callback=self.parse_article)
        
        if self.config['args']['source'] == 'defaultSearchlUrl':
            results = [int(s) for s in response.css('.flt-registros::text').extract()[0].split() if s.isdigit()][0]
            self.logger.info('Total of Results: %d', results)
            pages = int(math.ceil(results / 10))
            self.logger.info('Total of Pages: %d', pages)
            
            self.config['args']['source'] = 'ajaxSearchUrl'
            
            for p in range(1, pages+1):
                self.config['args']['page'] = p
                yield scrapy.Request(self.generateUrl(self.config['args']), callback=self.parse_article_urls)


    def parse_article_urls(self, response):
        for i, url in enumerate(response.css('.link-title::attr(href)').extract()):
            yield scrapy.Request(url, meta={'index': i}, callback=self.parse_article)


    def parse_article(self, response):
        l = NewsArticleItemLoader(item=NewsArticleItem(), response=response)

        l.add_css('name', 'h1::text')
        l.add_css('author', '.autores::text')
        l.add_css('description', '.definicao::text')
        l.add_css('datePublished', '.data::text')
        l.add_css('dateModified', '.data::text')
        # l.add_css('keywords', '[itemprop=keywords] a::text')
        l.add_css('articleBody', '.conteudo-materia')
        l.add_value('url', response.url)

        yield l.load_item()


    def generateUrl(self, values):
        url_parts = list(urlparse.urlparse(self.config[values['source']]))
        params = {}
        
        if values['source'] == 'defaultSearchlUrl':
            if values['query']:
                params['q'] = (values['query'])
            if values['editorial']:
                params['editoria[]'] = (values['editorial'])
            if values['subject']:
                params['assunto[]'] = (values['subject'])
            if values['when']:
                params['quando'] = (values['when'])
            if values['contentType']:
                params['tipo_conteudo'] = (values['contentType'])

        if values['source'] == 'ajaxSearchUrl':
            params = {
                'config[busca][page]': values['page'],
                'config[busca][params]': []
            }
            
            if values['query']:
                params['config[busca][params]'].append('q=' + values['query'])
            if values['editorial']:
                params['config[busca][params]'].append('&editoria[]=' + values['editorial'])
            if values['subject']:
                params['config[busca][params]'].append('&assunto[]=' + values['subject'])
            if values['when']:
                params['config[busca][params]'].append('&quando=' + values['when'])
            
            params['config[busca][params]'] = '&'.join(params['config[busca][params]'])
        
        url_parts[4] = urlencode(params)
        
        return urlparse.urlunparse(url_parts)



# -*- coding: utf-8 -*-

import math
try:
    import urlparse
    from urllib import urlencode
except: # Python 3
    import urllib.parse as urlparse
    from urllib.parse import urlencode
import scrapy
from scrapy.spiders import CrawlSpider, Rule
from scrapy.loader import ItemLoader
from scrapy.exceptions import CloseSpider
from ze.items.newsarticle import NewsArticleItem, NewsArticleItemLoader

class FolhaDeSaoPauloSpider(scrapy.Spider):

    name = 'folhadesp'
    allowed_domains = ['folha.uol.com.br']
    config = {
        'defaultSearchlUrl': '',
        'ajaxSearchUrl': '',
        'args': None
    }


    def __init__(self, url=None, query=None, editorial=None, subject=None, when=None, contentType=None, *args, **kwargs):
        super(FolhaDeSaoPauloSpider, self).__init__(*args, **kwargs)
        
        self.config['args'] = {
            'source': 'url', 
            'url': url, 
            'query': query, 
            'editorial': editorial, 
            'subject': subject, 
            'when': when, 
            'contentType': contentType, 
        }
        
        if url:
            self.start_urls = [self.config['args']['url']]
        else:
            self.config['args']['source'] = 'defaultSearchlUrl'
            self.start_urls = self.generateUrls(self.config['args'])


    def parse(self, response):
        if self.config['args']['source'] == 'url':
            yield scrapy.Request(self.config['args']['url'], callback=self.load_article_item)
        
        if self.config['args']['source'] == 'defaultSearchlUrl':
            raise NotImplemented
            results = [int(s) for s in response.css('.flt-registros::text').extract()[0].split() if s.isdigit()][0]
            self.logger.info('Total of Results: %d', results)
            pages = int(math.ceil(results / 10))
            self.logger.info('Total of Pages: %d', pages)
            
            self.config['args']['source'] = 'ajaxSearchUrl'
            
            for p in range(1, pages+1):
                self.config['args']['page'] = p
                yield scrapy.Request(self.generateUrls(self.config['args']), callback=self.load_article_item_urls)


    def load_article_item_urls(self, response):
        raise NotImplemented
        for i, url in enumerate(response.css('.link-title::attr(href)').extract()):
            yield scrapy.Request(url, meta={'index': i}, callback=self.load_article_item)


    def load_article_item(self, response):
        l = NewsArticleItemLoader(item=NewsArticleItem(), response=response)

        l.add_css('name', '[itemprop=headline]::text')
        l.add_fallback_css('name', '[itemprop="alternativeHeadline"]::attr(content)')
        l.add_fallback_css('name', 'article header h1::text')
        l.add_css('image', '[itemprop="image"]::attr(content)')
        l.add_css('author', '[itemprop=author]::text')
        l.add_fallback_css('author', '.author p::text')
        l.add_css('description', '[itemprop=description]::text')
        l.add_css('datePublished', '[itemprop=datePublished]::text')
        l.add_fallback_css('datePublished', 'article time::text')
        l.add_css('dateModified', '[itemprop=dateModified]::text')
        l.add_css('keywords', '[itemprop="keywords"]::attr(content)')
        l.add_css('articleBody', '[itemprop=articleBody]')
        l.add_fallback_css('articleBody', '.content')
        # TODO: Add publisher
        l.add_value('url', response.url)
        
        if 'blog' in response.url:
            l.add_value('sources_types', ('portal', 'blog'))
        else:
            l.add_value('sources_types', ('portal'))

        yield l.load_item()


    def generateUrls(self, values):
        raise NotImplemented



from pip.req import parse_requirements
from setuptools import setup, find_packages

setup(
    name = 'ze-the-scraper',
    version = '0.0.8.dev1',
    url = 'http://github.com/labic/ze-the-scraper',
    description = 'Scaper to lager portal of news in Brazil.',
    keywords = ['scrapy scraper spider crawler brazil news estadao veja folha-de-sp'],
    author = 'GustavoRPS',
    author_email = 'email+labic.net@gustavorps.net',
    license = 'MIT',
    packages = find_packages(),
    entry_points = {'scrapy': ['settings = ze.settings']},
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
    
        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
    
        # Pick your license as you wish (should match "license" above)
         'License :: OSI Approved :: Academic Free License (AFL)',
    
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.4',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
    ],
)



# -*- coding: utf-8 -*-

import dateparser
import logging
logger = logging.getLogger(__name__)

class CommonProcessor():

    @staticmethod
    def process_date_time(value):

        value = value.replace('Atualizado:', '') \
                    .replace(' | ', ' ') \
                    .replace('h', ':') \
                    .replace('h ', ':') \
                    .replace(', ', '') \
                    .replace('  ', ' ') \
                    .strip()
        date_formats = (
            '%d %B %Y %H:%M',
            '%d de %B de %Y %Hh%M',
            '%d/%m/%Y %H:%M',
        )
        
        try:
            return dateparser.parse(value, 
                        date_formats=['%d %B %Y %H:%M'], 
                        languages=['pt']) \
                        .isoformat()
        except Exception as e:
            logger.warning('Date %s not processed with none of formats: %s' % (value, ', '.join(date_formats)))
            pass

        return value

    # @staticmethod
    # def process_clean(value):
    #     values_to_clean = {
    #         ,
    #         ',': '',
    #         ' - ': '',
    #         'por ': '',
    #     }



# -*- coding: utf-8 -*-
import ze.utils.file

# Scrapy settings for ze module
#
# For simplicity, this file contains only settings considered important or
# commonly used. You can find more settings consulting the documentation:
#
#     http://doc.scrapy.org/en/latest/topics/settings.html
#     http://scrapy.readthedocs.org/en/latest/topics/downloader-middleware.html
#     http://scrapy.readthedocs.org/en/latest/topics/spider-middleware.html

PROJECT_NAME = 'ze-the-scraper'
SPIDER_MODULES = ['ze.spiders']
NEWSPIDER_MODULE = 'ze.spiders'

DUPEFILTER_DEBUG = True

# Crawl responsibly by identifying yourself (and your website) on the user-agent
USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/56.0.2924.76 Chrome/56.0.2924.76 Safari/537.36'

# Configure maximum concurrent requests performed by Scrapy (default: 16)
CONCURRENT_REQUESTS=32

# Configure a delay for requests for the same website (default: 0)
# See http://scrapy.readthedocs.org/en/latest/topics/settings.html#download-delay
# See also autothrottle settings and docs
# DOWNLOAD_DELAY=3
# The download delay setting will honor only one of:
CONCURRENT_REQUESTS_PER_DOMAIN=16
CONCURRENT_REQUESTS_PER_IP=16

# Disable cookies (enabled by default)
# COOKIES_ENABLED=False

# Disable Telnet Console (enabled by default)
#TELNETCONSOLE_ENABLED=False

# Override the default request headers:
#DEFAULT_REQUEST_HEADERS = {
#   'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
#   'Accept-Language': 'en',
#}

# Enable or disable spider middlewares
# See http://scrapy.readthedocs.org/en/latest/topics/spider-middleware.html
# SPIDER_MIDDLEWARES = {
#     'ze.middlewares.somemiddkeware': 100,
# }

# Enable or disable downloader middlewares
# See http://scrapy.readthedocs.org/en/latest/topics/downloader-middleware.html
DOWNLOADER_MIDDLEWARES = {
#     'rotating_proxies.middlewares.RotatingProxyMiddleware': 610,
#     'rotating_proxies.middlewares.BanDetectionMiddleware': 620,
}

# ROTATING_PROXY_LIST = ze.utils.file.load_lines('./proxies-list.txt')

GOOGLE_CLOUD_ENABLED = True
# Google Cloud Application Credentions used for many pipelines
GOOGLE_APPLICATION_CREDENTIALS_JSON = ''
# Enable or disable extensions
# See http://scrapy.readthedocs.org/en/latest/topics/extensions.html
EXTENSIONS = {
    'ze.extensions.google.GoogleCloud': 10
}

# MongoDB pipeline configuration
MONGO_URI = None
MONGO_DATABASE = None

# Configure item pipelines
# See http://scrapy.readthedocs.org/en/latest/topics/item-pipeline.html
ITEM_PIPELINES = {
    'ze.pipelines.GooglePubSubPipeline': 300,
    # 'ze.pipelines.MongoPipeline': 310,
}

# Enable and configure the AutoThrottle extension (disabled by default)
# See http://doc.scrapy.org/en/latest/topics/autothrottle.html
# NOTE: AutoThrottle will honour the standard settings for concurrency and delay
#AUTOTHROTTLE_ENABLED=True
# The initial download delay
#AUTOTHROTTLE_START_DELAY=5
# The maximum download delay to be set in case of high latencies
#AUTOTHROTTLE_MAX_DELAY=60
# Enable showing throttling stats for every response received:
#AUTOTHROTTLE_DEBUG=False

# Enable and configure HTTP caching (disabled by default)
# See http://scrapy.readthedocs.org/en/latest/topics/downloader-middleware.html#httpcache-middleware-settings
# HTTPCACHE_ENABLED=True
#HTTPCACHE_EXPIRATION_SECS=0
# HTTPCACHE_DIR='httpcache'
#HTTPCACHE_IGNORE_HTTP_CODES=[]
# HTTPCACHE_STORAGE='scrapy.extensions.httpcache.FilesystemCacheStorage'



import os, logging

logger = logging.getLogger(__name__)

class GoogleCloud(object):
    credentials_json_file = '../service-account.json'
    
    @classmethod
    def from_crawler(cls, crawler):
        return cls(settings=crawler.settings)

    def __init__(self, settings):
        self.google_cloud_enabled = settings.getbool('GOOGLE_CLOUD_ENABLED')
        
        if self.google_cloud_enabled:
            credentials_json = settings.get('GOOGLE_APPLICATION_CREDENTIALS_JSON')
            if credentials_json:
                with open(self.credentials_json_file, 'w') as outfile:
                    outfile.write(credentials_json)
                        
                os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = self.credentials_json_file
                logger.info('Google Cloud extensions inited with success')
            else:
                logger.error('GOOGLE_APPLICATION_CREDENTIALS_JSON not is set in settings')
                settings.set('GOOGLE_CLOUD_ENABLED', False)
        else:
            logger.info('GOOGLE_CLOUD_ENABLED is False')


    def close_spider(self, spider):
        if self.google_cloud_enabled:
            os.remove(self.credentials_json_file)
